The spinlock region_lock is held while calling mark_region which can
sleep.

The first section of this patch fixes this by dropping the spinlock 
before calling that function.

The second section...

From: Jonathan E Brassow <jbrassow@redhat.com>
Signed-Off-By: Alasdair G Kergon <agk@redhat.com>

[Submitted upstream 2005/11/18]

Index: linux-2.6.14-rc2/drivers/md/dm-raid1.c
===================================================================
--- linux-2.6.14-rc2.orig/drivers/md/dm-raid1.c	2005-10-26 20:14:27.000000000 +0100
+++ linux-2.6.14-rc2/drivers/md/dm-raid1.c	2005-10-26 20:23:03.000000000 +0100
@@ -376,16 +376,18 @@ static void rh_inc(struct region_hash *r
 	read_lock(&rh->hash_lock);
 	reg = __rh_find(rh, region);
 
+	spin_lock_irq(&rh->region_lock);
 	atomic_inc(&reg->pending);
 
-	spin_lock_irq(&rh->region_lock);
 	if (reg->state == RH_CLEAN) {
-		rh->log->type->mark_region(rh->log, reg->key);
-
 		reg->state = RH_DIRTY;
 		list_del_init(&reg->list);	/* take off the clean list */
-	}
-	spin_unlock_irq(&rh->region_lock);
+		spin_unlock_irq(&rh->region_lock);
+
+		rh->log->type->mark_region(rh->log, reg->key);
+	} else
+		spin_unlock_irq(&rh->region_lock);
+
 
 	read_unlock(&rh->hash_lock);
 }
@@ -408,21 +410,17 @@ static void rh_dec(struct region_hash *r
 	reg = __rh_lookup(rh, region);
 	read_unlock(&rh->hash_lock);
 
+	spin_lock_irqsave(&rh->region_lock, flags);
 	if (atomic_dec_and_test(&reg->pending)) {
-		spin_lock_irqsave(&rh->region_lock, flags);
-		if (atomic_read(&reg->pending)) { /* check race */
-			spin_unlock_irqrestore(&rh->region_lock, flags);
-			return;
-		}
 		if (reg->state == RH_RECOVERING) {
 			list_add_tail(&reg->list, &rh->quiesced_regions);
 		} else {
 			reg->state = RH_CLEAN;
 			list_add(&reg->list, &rh->clean_regions);
 		}
-		spin_unlock_irqrestore(&rh->region_lock, flags);
 		should_wake = 1;
 	}
+	spin_unlock_irqrestore(&rh->region_lock, flags);
 
 	if (should_wake)
 		wake();
